# Copyright (c) 2010 Aldo Cortesi
# Copyright (c) 2010, 2014 dequis
# Copyright (c) 2012 Randall Ma
# Copyright (c) 2012-2014 Tycho Andersen
# Copyright (c) 2012 Craig Barnes
# Copyright (c) 2013 horsik
# Copyright (c) 2013 Tao Sauvage
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import os
import socket
import subprocess
import re
import random
from datetime import datetime
from Xlib import display as xdisplay

from typing import Callable

from libqtile import bar, layout, widget
from libqtile.widget import Chord
from libqtile.config import Click, Drag, Group, Key, Match, Screen, KeyChord
from libqtile.lazy import lazy
from libqtile.utils import guess_terminal
from libqtile.log_utils import logger
from libqtile import hook

@hook.subscribe.startup_once
def autostart():
    home = os.path.expanduser('~/.config/qtile/autostart.sh')
    subprocess.Popen([home])

mod = "mod4"
alt = "mod1"
terminal = "kitty"
kbd_lights = ["off", "low", "med", "high"]
current_kbd_light = [0]
current_language = ["en"]

def kbd_brightness_up(qtile):
    idx = current_kbd_light[0]
    current_kbd_light[0] = min(len(kbd_lights) - 1, idx+1)
    value = kbd_lights[current_kbd_light[0]]
    qtile.spawn(f"asusctl -k {value}")

def kbd_brightness_down(qtile):
    idx = current_kbd_light[0]
    current_kbd_light[0] = max(0, idx-1)
    value = kbd_lights[current_kbd_light[0]]
    qtile.spawn(f"asusctl -k {value}")

def toggle_language(qtile):
    if current_language[0] == "en":
        qtile.spawn("setxkbmap -layout th")
        current_language[0] = "th"
    else:
        qtile.spawn("setxkbmap -layout us -variant altgr-intl")
        current_language[0] = "en"

def get_current_window_info(qtile):
    logger.warning(qtile.current_window.info())

@hook.subscribe.client_new
def new_client(client):
    if "Unity" in client.get_wm_class() and "Unity" == client.name:
        padding = 10
        x = client.group.screen.width - client.width - padding
        y = client.group.screen.height - client.height - padding - 22
        client.set_position_floating(x,y)

keys = [
    # A list of available commands that can be bound to keys can be found
    # at https://docs.qtile.org/en/latest/manual/config/lazy.html
    # Switch between windows
    # .when(wm_class != 'kitty')
    # Key([mod], "h", lazy.function(move_focus, direction='left'), desc="Move focus to left"),
    # Key([mod], "h", lazy.layout.left().when(focused=Match(wm_class=re.compile('(?![Ee]macs)'))), desc="Move focus to left"),
    # Key([mod], "j", lazy.layout.down().when(focused=Match(wm_class=re.compile('(?![Ee]macs)'))), desc="Move focus to left"),
    # Key([mod], "k", lazy.layout.up().when(focused=Match(wm_class=re.compile('(?![Ee]macs)'))), desc="Move focus to left"),
    # Key([mod], "l", lazy.layout.right().when(focused=Match(wm_class=re.compile('(?![Ee]macs)'))), desc="Move focus to left"),
    # Key([alt], "tab", lazy.layout.next(), desc="Move window focus to other window"),

    Key([mod], "o", lazy.layout.normalize(), desc="Move window focus to other window"),
    Key([alt], "tab", lazy.layout.previous(), desc="Move window focus to other window"),
    Key([mod], "tab", lazy.screen.toggle_group(), desc="Toggle Group"),
    # Eventually we should switch back to emacs app launcher
    Key([mod], "space", lazy.spawn("rofi -show drun -theme Arc-Dark.rasi"), desc="Open Rofi"),

    KeyChord([mod], "b", [
                 Key([], "c", lazy.spawn("connect-bt", shell=True)),
                 Key([], "d", lazy.spawn("bluetoothctl disconnect F8:5B:6E:2A:6A:CF &>/dev/null",
                                          shell=True)),
                 Key([], "r", lazy.spawn("bluetoothctl scan on && sleep 1 && bluetoothctl connect F8:5B:6E:2A:6A:CF && sleep 2 && bluetoothctl scan off",
                                          shell=True)),
                 Key([], "t", lazy.spawn("toggle-bt-codec", shell=True)),
              ]),
    # Move windows between left/right columns or move up/down in current stack.
    # Moving out of range in Columns layout will create new column.
    Key([mod, "shift"], "h", lazy.layout.shuffle_left(), desc="Move window to the left"),
    Key([mod, "shift"], "l", lazy.layout.shuffle_right(), desc="Move window to the right"),
    Key([mod, "shift"], "j", lazy.layout.shuffle_down(), desc="Move window down"),
    Key([mod, "shift"], "k", lazy.layout.shuffle_up(), desc="Move window up"),
    # Grow windows. If current window is on the edge of screen and direction
    # will be to screen edge - window would shrink.
    Key([mod, "control"], "h", lazy.layout.grow_left(), desc="Grow window to the left"),
    Key([mod, "control"], "l", lazy.layout.grow_right(), desc="Grow window to the right"),
    Key([mod, "control"], "j", lazy.layout.grow_down(), desc="Grow window down"),
    Key([mod, "control"], "k", lazy.layout.grow_up(), desc="Grow window up"),

    Key([mod, "control"], "u", lazy.spawn("pkill -9 Unity"), desc="Kill Unity"),
    Key([mod], "comma", lazy.screen.prev_group(), desc="Move to previous group"),
    Key([mod], "period", lazy.screen.next_group(), desc="Move to next group"),
    Key([mod, "shift"], "comma", lazy.prev_screen(), desc="Move focus to left screen"),
    Key([mod, "shift"], "period", lazy.next_screen(), desc="Move focus to left screen"),
    Key([mod], "w", lazy.next_layout(), desc="Toggle between layouts"),
    Key([mod, "control"], "w", lazy.function(get_current_window_info), desc="Get window info"),
    Key([mod], "s", lazy.spawn("flameshot gui"), desc="Flameshot screenshot"),
    Key([mod], "m", lazy.spawn("/home/joe/.local/scripts/trackball.sh", shell=True)),

    # TODO: Figure out another binding for this
    # Key([mod], "n", lazy.layout.normalize(), desc="Reset all window sizes"),

    # Toggle between split and unsplit sides of stack.
    # Split = all windows displayed
    # Unsplit = 1 window displayed, like Max layout, but still with
    # multiple stack panes
    Key([mod, "control", "shift"], "l",
        lazy.spawn("slock | systemctl suspend", shell=True), desc="Slock and suspend"),
    Key(
        [mod, "shift"],
        "Return",
        lazy.layout.toggle_split(),
        desc="Toggle between split and unsplit sides of stack",
    ),
    Key([mod], "Return", lazy.spawn(terminal), desc="Launch terminal"),
    # Toggle between different layouts as defined below
    Key([mod], "c", lazy.window.center(), desc="Center focused window"),
    # Key([mod], "Tab", lazy.next_layout(), desc="Toggle between layouts"),
    Key([mod], "q", lazy.window.kill(), desc="Kill focused window"),
    Key(
        [mod],
        "f",
        lazy.window.toggle_fullscreen(),
        desc="Toggle fullscreen on the focused window",
    ),
    Key([mod], "t", lazy.window.toggle_floating(), desc="Toggle floating on the focused window"),
    Key([mod, "control"], "r", lazy.reload_config(), desc="Reload the config"),
    Key([mod, "control"], "q", lazy.shutdown(), desc="Shutdown Qtile"),
    Key([mod, alt], "space", lazy.function(toggle_language), desc="Toggle Language Layout"),
    Key([mod], "r", lazy.spawncmd(), desc="Spawn a command using a prompt widget"),
    Key([], "XF86AudioLowerVolume", lazy.spawn("amixer sset Master 5%-")),
    Key([], "XF86AudioRaiseVolume", lazy.spawn("amixer sset Master 5%+")),
    Key([], "XF86Launch1", lazy.spawn("playerctl play-pause")),
    Key([], "XF86AudioPlay", lazy.spawn("playerctl play-pause")),
    Key([], "XF86MonBrightnessDown", lazy.spawn("brightnessctl set 5%-")),
    Key([], "XF86MonBrightnessUp", lazy.spawn("brightnessctl set +5%")),
    # This uses discrete values
    Key([], "XF86KbdBrightnessDown", lazy.function(kbd_brightness_down)),
    Key([], "XF86KbdBrightnessUp", lazy.function(kbd_brightness_up)),
    Key([mod, "control", "shift", alt], "1", lazy.window.set_position_floating(5, 28), desc="Put the window in the corner"),
]

pape_dir = '/home/joe/Pictures/Wallpapers/'
papes = [os.path.join(pape_dir, p) for p in os.listdir(pape_dir)]
if papes:
    pape = random.choice(papes)
else:
    pape = "/usr/share/backgrounds/default.png"

def get_num_monitors():
    number_of_monitors = 0
    try:
        x_display = xdisplay.Display()
        resources = x_display.screen().root.xrandr_get_screen_resources()

        for output in resources.outputs:
            monitor = x_display.xrandr_get_output_info(
                output, resources.config_timestamp)

            preferred = False
            if hasattr(monitor, "preferred"):
                preferred = monitor.preferred
            elif hasattr(monitor, "num_preferred"):
                preferred = monitor.num_preferred
            if preferred:
                number_of_monitors += 1
    except Exception as e:
        # always setup at least one monitor
        logger.error(e)
        return 1
    else:
        return number_of_monitors

group_box = widget.GroupBox()
current_layout = widget.CurrentLayout()
prompt = widget.Prompt()
window_name = widget.WindowName()
window_name2 = widget.WindowName()
chord = widget.Chord(chords_colors={"launch": ("#ff0000", "#ffffff"),}, name_transform=lambda name: name.upper())
systray = widget.Systray()
battery = widget.Battery() if socket.gethostname() != "puter" else widget.CPU()
clock = widget.Clock(format="%Y-%m-%d %a %I:%M %p")
volume = widget.Volume()

screens = [
    Screen(
        wallpaper=pape,
        wallpaper_mode='fill',
        top=bar.Bar([
                group_box, current_layout, prompt, window_name,
                systray, chord, volume, battery, clock,
            ],
            24,
            # border_width=[2, 0, 2, 0],  # Draw top and bottom borders
            # border_color=["ff00ff", "000000", "ff00ff", "000000"]  # Borders are magenta
        ),
    ),
]

groups = [Group(i) for i in "0123456789"]
num_monitors = get_num_monitors()
if num_monitors > 1:
    side_screen = Screen(
        wallpaper=pape,
        wallpaper_mode='fill',
        # bottom=bar.Bar(
        #     [
        #         # group_box, current_layout, prompt, window_name2, chord, volume, battery,
        #     ],
        #     24,
        #     # border_width=[2, 0, 2, 0],  # Draw top and bottom borders
        #     # border_color=["ff00ff", "000000", "ff00ff", "000000"]  # Borders are magenta
        # ),
    )
    screens.insert(0, side_screen)
    groups[0].screen_affinity = 0
    for g in groups[1:]:
        g.screen_affinity = 1

groups[0].label = '🎵'


# groups.insert(0, Group('1', screen_affinity=0, label='🎵'))

def go_to_group(name: str) -> Callable:
    def _inner(qtile) -> None:
        if num_monitors > 1:
            if name == '0':
                qtile.focus_screen(0)
            else:
                qtile.focus_screen(1)
        qtile.groups_map[name].toscreen()
    return _inner

def swap_windows(name: str) -> Callable:
    def _inner(qtile) -> None:
        other_windows = [*qtile.groups_map[name].windows]
        current_windows = [*qtile.current_group.windows]
        for win in current_windows:
            win.togroup(name)
        for win in other_windows:
            win.togroup(qtile.current_group.name)

    return _inner

for i in groups:
    key_name = i.name if i.name != "0" else "grave"
    keys.extend(
        [
            # mod1 + letter of group = switch to group
            Key(
                [mod],
                key_name,
                lazy.function(go_to_group(i.name)),
                desc="Switch to group {}".format(i.name),
            ),
            # mod1 + shift + letter of group = switch to & move focused window to group
            Key(
                [mod, "shift"],
                key_name,
                lazy.window.togroup(i.name, switch_group=False),
                desc="Switch to & move focused window to group {}".format(i.name),
            ),
            Key(
                [mod, "control"],
                key_name,
                lazy.function(swap_windows(i.name)),
                desc="Swap all windows with windows from N group".format(i.name),
            ),
            # Or, use below if you prefer not to switch to that group.
            # # mod1 + shift + letter of group = move focused window to group
            # Key([mod, "shift"], i.name, lazy.window.togroup(i.name),
            #     desc="move focused window to group {}".format(i.name)),
        ]
    )

# keys.append(Key([mod], 'm', lazy.function(go_to_music), desc="Music"))

layouts = [
    layout.Max(),
    layout.Columns(
        border_focus=["#999999"],
        border_width=1
    ),
    # Try more layouts by unleashing below layouts.
    # layout.Stack(num_stacks=2),
    # layout.Bsp(),
    # layout.Matrix(),
    # layout.MonadTall(),
    # layout.MonadWide(),
    # layout.RatioTile(),
    # layout.Tile(),
    # layout.TreeTab(),
    # layout.VerticalTile(),
    # layout.Zoomy(),
]

widget_defaults = dict(
    font="JetBrains Mono",
    fontsize=12,
    padding=5,
)

# Drag floating layouts.
mouse = [
    Drag([mod], "Button1", lazy.window.set_position_floating(), start=lazy.window.get_position()),
    Drag([mod], "Button3", lazy.window.set_size_floating(), start=lazy.window.get_size()),
    Click([mod], "Button2", lazy.window.bring_to_front()),
]

dgroups_key_binder = None
dgroups_app_rules = []  # type: list
follow_mouse_focus = True
bring_front_click = False
floats_kept_above = True
cursor_warp = False
floating_layout = layout.Floating(
    float_rules=[
        # Run the utility of `xprop` to see the wm class and name of an X client.
        *layout.Floating.default_float_rules,
        # This is the youtube music brave thing
        Match(wm_class="crx_cinhimbnkkaeohfgghhklpknlkffjgod"),
        Match(wm_class="p5py"),
        Match(wm_class="pygame"),
        Match(wm_class="gnome-calculator"),
        Match(wm_class="confirmreset"),  # gitk
        Match(wm_class="makebranch"),  # gitk
        Match(wm_class="maketag"),  # gitk
        Match(wm_class="ssh-askpass"),  # ssh-askpass
        Match(title="SimpleScreenRecorder"),  # ssh-askpass
        # Godot
        Match(title="Alert!"),
        Match(title="Please Confirm..."),
        Match(title="Create New Node"),
        Match(title="Select Frames"),
        Match(title="Open a file"),
        Match(title="Node Configuration Warning!"),
        Match(title="Save Scene As..."),
        Match(title="branchdialog"),  # gitk
        Match(title="pinentry"),  # GPG key password entry
        # Unity3D
        Match(wm_class="Unity", title="UnityEditor.AddComponent.AddComponentWindow"),
        Match(wm_class="Unity", title="UnityEditor.IconSelector"),
        Match(wm_class="Unity", title="UnityEditor.PopupWindow"),
        Match(wm_class="Unity", title="UnityEditor.PopupWindowWithoutFocus"),
        Match(wm_class="Unity", title="UnityEditor.AnnotationWindow"),
        Match(wm_class="Unity", title="Enable Android Auto-resolution?"),
        Match(wm_class="Unity", title="Select Material"),
        Match(wm_class="Unity", title="Select Preset"),
        Match(wm_class="Unity", title="Color"),
        Match(wm_class="Unity", title="Save Layout"),
        Match(wm_class="Unity", title="Sirenix.OdinInspector.Editor.TypeSelectorV2"),
        Match(wm_class="Unity", title=re.compile(r"^Sirenix.OdinInspector.Editor.EnumSelector.*")),
        # Match(wm_class="Unity", title=re.compile(r"^Select .*")),
    ]
)
auto_fullscreen = True
focus_on_window_activation = "smart"
reconfigure_screens = True

# If things like steam games want to auto-minimize themselves when losing
# focus, should we respect this or not?
auto_minimize = True

# When using the Wayland backend, this can be used to configure input devices.
wl_input_rules = None

# XXX: Gasp! We're lying here. In fact, nobody really uses or cares about this
# string besides java UI toolkits; you can see several discussions on the
# mailing lists, GitHub issues, and other WM documentation that suggest setting
# this string if your java app doesn't work correctly. We may as well just lie
# and say that we're a working one by default.
#
# We choose LG3D to maximize irony: it is a 3D non-reparenting WM written in
# java that happens to be on java's whitelist.
wmname = "LG3D"

logger.warning('Loaded: ' + str(datetime.now()))
